__version__ = '2.1.3'

### Version history
#
# 2.0   - Ruben Riddeström's BSc thesis work
# 2.1   - Lars' adaptions
# 2.1.1 - Updated README, corrected URL,
# 2.1.2 - Fixed content type on README for PyPi
# 2.1.3 - Updated unit tests and corrected Octave output
